/******** fastvid.c **********/ 
#include <OS.h> 
#include <PCI.h> 

#include <stdlib.h> 
#include <stdio.h> 

#include <unistd.h> /* ioctl() */ 
#include <fcntl.h> /* open()/close() */ 
#include <errno.h> 
#include <string.h> /* strerror() */ 

#include "mtrr_drv.h" 

void print_usage(void) 
{ 
  printf("fastvid enables/disables write combining 
    for a graphic frame buffer.\n"); 
  printf("Usage:\n"); 
  printf("  fastvid set|clear\n"); 
  printf("  fastvid set mem_base mem_len mem_type\n"); 
  printf("  fastvid clear mem_base\n"); 
  exit(0); 
} 

void 
find_frame_buffer(mtrr_drv_command_t* info) 
{ 

  int       i; 
  pci_info  h; 
  long      n; 

  for (n=0; get_nth_pci_info(n, &h) == B_OK; n++) 
  { 

    if (h.class_base != PCI_display &&
      !(h.class_base == PCI_early &&
        h.class_sub  == PCI_early_vga) ) 
        continue; 

    for (i = 0; i < 6; i++) 
    { 
      if (h.u.h0.base_register_flags [i] & PCI_address_space) 
        continue; 

      if (h.u.h0.base_register_sizes[i] < (1024*1024) ) 
        continue; 

      info->len = h.u.h0.base_register_sizes[i]; 
      info->base = h.u.h0.base_registers[i]; 

		//limit the size of the guessed frame buffer
//      if(info->len > 8*1024*1024)  
//		     info->len = 8*1024*1024; 


       return; // B_OK 
    } 
  } 
  printf("Can not find frame buffer\n"); 
  exit(2); 
  // info->len = info->base = 0; 
  //return B_ERROR; 

//      info->len = 2 * 1024 * 1024; 
//      info->base = 0xa0000000; 
} 

int main(int argc, char* argv[]) 
{ 
  mtrr_drv_command_t  command; 
  int        fd; 
  status_t   ret; 
  
system_info sys_inf;
cpuid_info id_inf;
uint32 eax = 0x00;
get_system_info(&sys_inf);
printf("cpu_info  : $%08x\n", sys_inf.cpu_type);
get_cpuid(&id_inf, eax, 0);
printf("cpuID_info: %12s\n", id_inf.eax_0.vendorid);

  if(argc<2) 
    print_usage(); 

  if(strcmp(argv[1], "set") == 0) 
  { 
    command.action = RANGE_SET; 

    switch(argc) 
    { 
    case 2: 
      find_frame_buffer(&command); 
      command.mem_type = WC_MEM; 
      break; 

    case 5: 
      command.base = strtoull(argv[2], NULL, 0); 
      command.len = strtoull(argv[3], NULL, 0); 
      command.mem_type = (MEM_TYPE)strtoul(argv[4], NULL, 0); 
      break; 

    default: 
      print_usage(); 
    } 
  } 
  else if(strcmp(argv[1], "clear") == 0) 
  { 
    command.action = RANGE_CLEAR; 

    switch(argc) 
    { 
    case 2: 
      find_frame_buffer(&command); 
      break; 

    case 3: 
      command.base = strtoull(argv[2], NULL, 0); 
      break; 

    default: 
      print_usage(); 
    } 
  } 
  else 
  { 
    print_usage(); 
  } 
  /* printf("%d %d %Lx %Lx\n", command.action, command.mem_type, 
       command.base, command.len); */ 

  errno = 0; 
  fd = open( "/dev/cpu/mtrr", O_RDWR); 
  if(fd == -1) 
  { 
    printf("Can't open /dev/cpu/mtrr, errno=%d (%s)\n", 
           errno, strerror(errno)); 
    exit(1); 
  } 

  ret = ioctl(fd, MTTR_DRV_COMMAND, &command, 
              sizeof(command)); 

  if(ret == B_OK) 
  { 
    printf("Success!\n"); 
  } 
  else 
  { 
    printf("Error: ioctl returned 0x%lx, errno=%d (%s)\n", 
           ret, errno, strerror(errno)); 
    exit(1); 
  } 

  return 0; 
} 
